function psi = gllim2gmmj(theta)
%%%%%%%%% Conversion of joint gmm parameters to gllim parameters %%%%%%%%%%
%%%% Author: Antoine Deleforge (July 2012) - antoine.deleforge@inria.fr %%%
% Description: Convert joint GMM parameters psi to GLLiM parameters theta
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Input %%%%
% - theta  (struct)       % GLLiM parameters
%   - theta.c (LxK)       % Gaussian means of T
%   - theta.Gamma (LxLxK) % Gaussian covariances of T
%   - theta.pi (1xK)      % Gaussian weights of T
%   - theta.A (DxLxK)     % Affine transformation matrices
%   - theta.b (DxK)       % Affine transformation vectors
%   - theta.Sigma (DxDxK) % Error covariances
%%%% Output %%%%
% - psi (struct)            % Joint GMM parameters
%   - psi.mu (L+DxK)        % Gaussian means
%   - psi.Sigma (L+DxL+DxK) % Gaussian covariances
%   - psi.weight (1xK)      % Gaussian weights
% - L (int)                 % Latent variable dimensionality (T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2012 Antoine Deleforge
%
% This file is part of GLLiM v1.0.
%
% GLLiM v1.0 is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% GLLiM v1.0 is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with GLLiM v1.0. If not, see <https://www.gnu.org/licenses/>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[D,K]=size(theta.b);
L=size(theta.c,1);
psi.weight=theta.pi;
psi.mu=zeros(L+D,K);
psi.Sigma=zeros(L+D,L+D,K);
for k=1:K
    Ak=reshape(theta.A(:,:,k),[D,L]);
    ck=theta.c(:,k);
    bk=theta.b(:,k);
    Gammak=reshape(theta.Gamma(:,:,k),[L,L]);
    Sigmak=reshape(theta.Sigma(:,:,k),[D,D]);
    psi.mu(:,k)=[ck;Ak*ck+bk];
    psi.Sigma(:,:,k)=[Gammak,Gammak*Ak';Ak*Gammak,Sigmak+Ak*Gammak*Ak'];
end
end