function theta = gmmj2gllim(psi,L)
%%%%%%%%% Conversion of gllim parameters to joint gmm parameters %%%%%%%%%%
%%%% Author: Antoine Deleforge (July 2012) - antoine.deleforge@inria.fr %%%
% Description: Convert joint GMM parameters psi to GLLiM parameters theta
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Input %%%%
% - psi (struct)            % Joint GMM parameters
%   - psi.mu (L+DxK)        % Gaussian means
%   - psi.Sigma (L+DxL+DxK) % Gaussian covariances
%   - psi.weight (1xK)      % Gaussian weights
% - L (int)                 % Low dimensional variable dimensionality (T)
%%%% Output %%%%
% - theta  (struct)       % GLLiM parameters
%   - theta.c (LxK)       % Gaussian means of T
%   - theta.Gamma (LxLxK) % Gaussian covariances of T
%   - theta.pi (1xK)      % Gaussian weights of T
%   - theta.A (DxLxK)     % Affine transformation matrices
%   - theta.b (DxK)       % Affine transformation vectors
%   - theta.Sigma (DxDxK) % Error covariances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2012 Antoine Deleforge
%
% This file is part of GLLiM v1.0.
%
% GLLiM v1.0 is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% GLLiM v1.0 is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

% You should have received a copy of the GNU General Public License
% along with GLLiM v1.0. If not, see <https://www.gnu.org/licenses/>
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[DL,K]=size(psi.mu);
D=DL-L;
theta.c=psi.mu(1:L,:);
theta.Gamma=psi.Sigma(1:L,1:L,:);
theta.pi=psi.weight; 
theta.A=zeros(D,L,K);    
if(L==0)
   theta.b=psi.mu;
   theta.Sigma=psi.Sigma;
else
   theta.b=zeros(D,K);    
   theta.Sigma=zeros(D,D,K);           
   for k=1:K
       muxk=reshape(psi.mu(1:L,k),L,1);
       muyk=reshape(psi.mu(L+1:L+D,k),D,1);       
       Sxxk=reshape(psi.Sigma(1:L,1:L,k),L,L);
       Sxyk=reshape(psi.Sigma(1:L,L+1:L+D,k),L,D);
       Syyk=reshape(psi.Sigma(L+1:L+D,L+1:L+D,k),D,D);
       Ak=Sxyk'/Sxxk;
       theta.A(:,:,k)=Ak;
       theta.b(:,k)=muyk-Ak*muxk;
       theta.Sigma(:,:,k)=Syyk-Ak*Sxyk;
   end
end
end